RESEARCH_SESSION_CONFIG = [
    {
        'construction': {
            'model': None,
            'optimizer': None,
            'criterion': None,
            'watch_test': None,
            'research_scheme': None
        },
        'session': {
            'train_dataloader': None,
            'test_dataloader': None,
            'dataset_name': None,
            'epochs': None,
            'iteration_modulo': None
        }

    }
]
